//
//  YHFileListViewController.swift
//  galaxy
//
//  Created by davidhuangA on 2024/3/17.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit
import Photos
/*
 文件清单
 */

class YHFileListViewController: YHBaseViewController {
    
    var orderId : Int?
    private let certificateReqVM : YHCertificateViewModel = YHCertificateViewModel()
    
    
    
    private lazy var headView : YHFileListHeaerView = {
        let view = YHFileListHeaerView()
        view.backgroundColor = .clear
        return view
    }()
    
    private lazy var myScrollView : UIScrollView = {
        let view = UIScrollView()
        view.backgroundColor = .white
        return view
    }()
    
    
    private lazy var myContentView : YHFileListContentView = {
        let view = YHFileListContentView()
        view.backgroundColor = .white
        view.layer.cornerRadius = kCornerRadius6
        view.clipsToBounds = true
        return view
    }()
    
    
    let bottomView : YHFileListBottomView = YHFileListBottomView()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        loadData()
        loadData2()
    }
    
    
    private lazy var footLable : UILabel = {
        
        let footLable = UILabel()
        footLable.text = "*感谢您对银河移民的信赖，以上信息均用于香港优才签证申 请银河移民承诺对您所提供的信息将绝对保密"
        footLable.textColor = .white
        footLable.font = UIFont.PFSC_R(ofSize: 12)
        footLable.numberOfLines = 0
        footLable.lineBreakMode = .byWordWrapping
        return footLable
    }()
}

extension YHFileListViewController {
    func loadData() {
        if let orderID = UserDefaults.standard.value(forKey: "orderIdForPreview") {
            
            YHHUD.show(.progress(message: ""))
            let param = ["order_id":orderID,"node":"1","state":"1"]
            certificateReqVM.getCertificateFileListData(params: param) { success, error in
                YHHUD.hide()
                if success == true {
                    self.myContentView.dataModel = self.certificateReqVM.arrFileListData
                } else {
                    YHHUD.flash(message: error?.errorMsg ?? "请求出错")
                }
            }
        } else {
            printLog("error : orderID 为空")
        }
    }
    
    func loadData2() {
        if let orderID = UserDefaults.standard.value(forKey: "orderIdForPreview") {
            let param = ["order_id":orderID,"node":"1","state":"1"]
            certificateReqVM.getCertificateStatisticData(params: param) { success, error in
                if success == true {
                    self.headView.dataModel = self.certificateReqVM.certificateEntryModel
                } else {
                    YHHUD.flash(message: error?.errorMsg ?? "请求出错")
                }
            }
        } else {
            printLog("error : orderID 为空")
        }
    }
    
    
    func setupUI() {
        gk_navTitle = "文件清单"
        gk_navBarAlpha = 1.0
        gk_navBackgroundColor = .white
        
        view.backgroundColor = UIColor(hex:0x0D37C7, alpha: 1.0)
        
        view.addSubview(bottomView)
        bottomView.snp.makeConstraints { make in
            make.left.right.bottom.equalToSuperview()
            make.height.equalTo(YHFileListBottomView.viewH)
        }
        
        bottomView.block = { [self] flag in
            if flag == 1 {
                printLog("生成清单")
//                if let image = self.captureScrollView(scrollView: self.myScrollView) {
//                    saveImageToPhotosAlbum(image: image)
//                } else {
//                    YHHUD.flash(message: "截图失败~~~")
//                }
            }
        }
        
        myScrollView.backgroundColor = UIColor(hex:0x0D37C7, alpha: 1.0)
        
        view.addSubview(myScrollView)
        myScrollView.snp.makeConstraints { make in
            make.top.equalToSuperview().offset(k_Height_NavigationtBarAndStatuBar)
            make.left.right.equalToSuperview()
            make.bottom.equalTo(bottomView.snp.top)
        }
        
        myScrollView.addSubview(headView)
        headView.snp.makeConstraints { make in
            make.top.left.right.equalToSuperview()
            make.height.equalTo(YHFileListHeaerView.viewH)
        }
        
        myScrollView.addSubview(myContentView)
        myContentView.snp.makeConstraints { make in
            make.top.equalTo(217)
            make.left.equalTo(kMargin)
            make.width.equalTo(KScreenWidth - 2*kMargin)
        }
        
        myScrollView.addSubview(footLable)
        footLable.snp.makeConstraints { make in
            make.top.equalTo(myContentView.snp.bottom).offset(29)
            make.left.equalTo(28)
            make.width.equalTo(KScreenWidth - 2*28)
            make.bottom.equalToSuperview().offset(-41)
        }
        return
    }
    
    func captureScrollView(scrollView : UIScrollView) -> UIImage? {
        var image: UIImage? = nil
        let savedContentOffset = scrollView.contentOffset
        let savedFrame = scrollView.frame
        scrollView.contentOffset = .zero
        scrollView.frame = CGRect(x: 0, y: 0,
                                  width: scrollView.contentSize.width,
                                  height: scrollView.contentSize.height)
        
        UIGraphicsBeginImageContext(scrollView.frame.size)
        UIGraphicsBeginImageContextWithOptions(
            CGSize(width: scrollView.frame.size.width,
                   height: scrollView.frame.size.height),
            false,
            UIScreen.main.scale)
        scrollView.layer.render(in: UIGraphicsGetCurrentContext()!)
        image = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()
        
        print("contentSize == \(scrollView.contentSize)")
        scrollView.contentOffset = savedContentOffset
        scrollView.frame = savedFrame
        return image
        
    }
    
    
    func saveImageToPhotosAlbum(image: UIImage) {
        // 确保应用有权访问相册
        PHPhotoLibrary.requestAuthorization { status in
            if status == .authorized {
                // 保存图片到相册
                UIImageWriteToSavedPhotosAlbum(image, nil, nil, nil)
                YHHUD.flash(message: "保存成功")
            } else {
                YHHUD.flash(message: "范围图库权限不足")
            }
        }
    }
}
